// ENSAM 2005 ||| WORKSHOP INFOGRAPHIE

/////////////////////////////////////////////
//	MACROS :
/////////////////////////////////////////////

//	TEXTURES

#macro ma_texture( ccc, rrr )
	texture
	{	pigment	{ color rgbt ccc }
		finish
		{	ambient 	0.3
			diffuse		0.7
			specular	0.7
			roughness	0.01
			reflection	rrr
		}
	} 
#end

#macro texture_echiquier()
	texture
	{	pigment 
		{	checker
			color rgbt <1/4,1/4,1/4,0.8>
			color rgbt <3/4,3/4,3/4,0.0>
			scale <1, 1/64, 1>
			rotate <0,0,0>
			translate <1/2,0,1/2>
			
		}
		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
			//reflection 0.8
		}
	}
	// interior { ior 1.3 }
#end

#declare BOIS 	= 0;
#declare MARBRE = 1;
#declare GRANIT = 2;

#macro textures_diverses( choix )
	texture
	{	pigment 
		{	#if (choix = BOIS) 		wood 	#end
			#if (choix = MARBRE) 	marble 	#end
			#if (choix = GRANIT) 	granite #end
			color_map
			{
				[ 0.4 color rgbt <1,0,0,0.9> ]
				[ 0.5 color rgb <0,1,0> ]
				[ 0.6 color rgbt <0,0,1> ]
			}
			turbulence 0.1
			rotate <0,0,30>
			scale 1/8
		}
		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
			//reflection 0.8
		}
	}
	// interior { ior 1.3 }
#end

#macro texture_bump()
	normal
	{	
		bumps	1
		//waves 1/2
		scale 1/64
	}
#end

#declare GIF 	= 0;
#declare JPG	= 1;
#declare PNG	= 2;

#macro texture_image( nom, type_image )
	texture
	{	pigment
    	{	image_map
      		{
      			#if (type_image = GIF)	gif 	#end
      			#if (type_image = JPG)	jpeg 	#end
      			#if (type_image = PNG)	png 	#end
      			nom				
        		interpolate 2 		// lissage
        		once   				// pas de repetition
       		}
      		translate <-0.5,-0.5,0>	// centrage
     		scale 1/2				// taille
			
  		}
 		finish
		{	ambient 0.2
			diffuse 0.7
			specular 0.7
			roughness 0.01
		}
	}
#end

//	GEOMETRIE

#macro grand_tore( ccc )
	object
	{	torus {  0.5, 0.2  }
		ma_texture(  ccc, 0.0 ) 
	}
#end

#macro petit_tore( aaa, ccc )
	object
	{	torus {  0.2, 0.05  }
		translate <0.5,0,0>
		rotate <90,aaa,0>
		ma_texture( ccc, 0 ) 
	}
#end

#macro multi_tores( N)
	union
	{
		#local i = 0;
			#while ( i<N )	
			petit_tore( 360/N*i, < 	1-i/(N-1), 	// rouge de 1  0
									i/(N-1), 	// vert de 0  1
									0,  		// bleu 0
									0> 			// transparent 0
									)
			#local i=i+1;
		#end
	}
#end

#macro anneau_1()
	difference
	{	multi_tores( 12 )
		grand_tore( <0,0,0> )
	}
#end

#macro anneau_2()
	difference
	{	grand_tore( <0,0,0> )
		multi_tores( 12 )
		translate <0.5,0,0>
		rotate <90,0,0>
	}
#end

#macro une_geometrie()
	difference
	{	intersection
		{	box { <-1/2,-1/2,-1/2>, <1/2,1/2,1/2> scale 6/8 }
			sphere { <0,0,0>  1/2 }
		}
		cylinder { <0,-1/2-1,0>, <0,1/2+1,0>, 1/4 }
		cylinder { <-1/2-1,0,0>, <1/2+1,0,0>, 1/4 }
	}
#end

#macro objet_test()
	object
	{	une_geometrie()
		ma_texture( <1,0,0>, 0 )
		#if ( RENDU_FINAL = true )
			texture_echiquier()
			textures_diverses( GRANIT )
			texture_bump()
		#end
		texture_image( "artist2_on.gif", GIF )
	}
#end

//	VISUALISATION

#macro axes()
	cylinder { <-10,0,0>, <10,0,0>, 0.01 pigment { color rgb <1,0,0> } }
	cylinder { <0,-10,0>, <0,10,0>, 0.01 pigment { color rgb <0,1,0> } }
	cylinder { <0,0,-10>, <0,0,10>, 0.01 pigment { color rgb <0,0,1> } }
#end

#macro sol( taille, translation, gris)
	object
	{	box { <-taille,-0.01,-taille>, <taille,0.01,taille> }
		ma_texture( <gris,gris,gris>, 0 )
		translate <0,translation, 0>
	}
#end

#declare OX 	= 0;
#declare OY 	= 1;
#declare OZ 	= 2;
#declare AXO_1 	= 3;
#declare AXO_2 	= 4;
#declare AXO_3 	= 5;
#declare AXO_4 	= 6;

#macro axonometrie ( vvv, zoom )
	#if (vvv = OX) camera { orthographic location <-1,0,0> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = OY) camera { orthographic location <0,-1,0> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = OZ) camera { orthographic location <0,0,-1> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = AXO_1) camera { orthographic location <-1,1,-1> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = AXO_2) camera { orthographic location < 1,1,-1> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = AXO_3) camera { orthographic location < 1,1, 1> look_at <0,0,0> scale 1/zoom } #end
	#if (vvv = AXO_4) camera { orthographic location <-1,1, 1> look_at <0,0,0> scale 1/zoom } #end
#end

#macro pers ( oeil, cible )
	camera { location oeil look_at cible }
#end

#macro eclairage_standard( nombre )
	#if (nombre=1)
		light_source { <-1,1,-1> color rgb <1,1,1> }	
	#end
	#if (nombre=2)	
		light_source { <-1,1,-1> color rgb <1,1,1> }	
		light_source { < 1,1,-1> color rgb <1,1,1> }
	#end
	#if (nombre=3)	
		light_source { <-1,1,-1> color rgb <1,1,1> }	
		light_source { < 1,1,-1> color rgb <1,1,1> }
		light_source { < 0,2, 2> color rgb <1,1,1> }
	#end
#end

#macro fond( gris )
	background { color rgb gris }
#end

//	UN PEU DE MATHS :

#macro segment ( p0, p1, tt )
	(1-tt)*p0 + tt*p1;
#end

#macro cercle( rayon, tt, translation )
	#local xx = rayon * cos( 2*pi*tt );
	#local zz = rayon * sin( 2*pi*tt );
	centre + <xx,0,zz>;
#end

#macro helice( _r0, _r1, tt, h0, h1, N )
	#if (tt<0.5)	#local rr = (1-tt)*_r0 + tt*_r1;	#end
	#if (tt>=0.5)	#local rr = (1-tt)*_r1 + tt*_r0;	#end
	#local xx = rr * cos( 2*N*pi*tt );
	#local zz = rr * sin( 2*N*pi*tt );
	#local yy = (1-tt)*h0 + tt*h1;
	<xx,yy,zz>;
#end

#macro courbe( N )
	#local i=0; #while (i<N)
		#local p = helice(  0.2, 2.0, i/N, -2.0, 2.0, 4 )
		sphere { p, 0.02 pigment { color rgb <0,1,0> } }
	#local i=i+1; #end
#end

/*
	SPLINES:
	polylignes ou polycourbes issues de la concatenation de 
	segments, de quadriques, de cubiques  ;
*/

#macro lin(a, p0, p1)		// p0 et p1 sont deux points de l'espace,
	(1-a)*p0 + 				// a est suppose varier dans l'intervalle [0,1]
	    a*p1				// le point calcule se deplacera de p0 a p1
#end

#macro quad(a, p0, p1, p2)
	(1-a)*(1-a)*p0 + 		// noter la forme: 1*k0*p0 + 2*k1*p1 + 1*k2*p2
	  2*(1-a)*a*p1 + 		//							1 2 1
	        a*a*p2
#end

#macro cub(a, p0, p1, p2, p3)
	(1-a)*(1-a)*(1-a)*p0 + 	// noter la forme: 1*k0*p0 + 3*k1*p1 + 3*k2*p2 + 1*k3*p3
	  3*(1-a)*(1-a)*a*p1 + 	// 							1 3 3 1
	      3*(1-a)*a*a*p2 + 	// forme des coefficients du triangle de Pascal, au dela
	            a*a*a*p3	// on aurait: 1 4 6 4 1, puis 1 5 10 10 5 1, etc...
#end


#macro fonction_spline(	pp, degre, a )
	// pp : tableau de (n+1) points ; degre : lin, para ou cub, 1, 2, 3 ; a dans [0,1]						
	#local n = dimension_size( pp,1)-1;		// nb segments
	#if (degre = 1 )
		#local ppp = pp
		#local nn = n;
	#end	
	#if (degre = 2 )
		#local _n = dimension_size( pp,1);
		#local _nn = _n+2;
		#local ppp = array[_nn]
		#local i=0; #while (i<_n)
			#local ppp[i+1] = pp[i];
		#local i=i+1; #end
		#local ppp[0] = pp[0];
		#local ppp[_nn-1] = pp[_n-1];
		#local nn = _nn-1;
	#end
	#if (degre = 3 )
		#local _n = dimension_size( pp,1);
		#local _nn = _n+4;
		#local ppp = array[_nn]
		#local i=0; #while (i<_n)
			#local ppp[i+2] = pp[i];
		#local i=i+1; #end
		#local ppp[0] = pp[0];
		#local ppp[1] = pp[0];
		#local ppp[_nn-2] = pp[_n-1];
		#local ppp[_nn-1] = pp[_n-1];
		#local nn = _nn-1;
	#end				
	#declare Il=0;								// on recherche le segment I:
	#local   i=0;								// 
	#while (i < nn-degre+1)						// 
		#if ( (i/nn <= a) & (a <= (i+1)/nn) ) 	// 
			#declare Il = i;					// 
		#end									//
		#local i=i+1;							//
	#end										// on connait maintenant le segment I
	#local aa = nn*a - Il;						// on calcule le a local: aa dans [0,1]
	#local p = <0,0,0>;						// initialisation du point cherche
	#switch (degre)								// suivant le degre:
		#case (1)								// 	cas lineaire:
			#local q0 = ppp[Il];					//
			#local q1 = ppp[Il+1];				//
			#local p = lin(aa, q0, q1);		// 
		#break
		#case (2)								// 	cas quadratique:
			#local q0 = (ppp[Il]   + ppp[Il+1])/2;	//
			#local q1 =  ppp[Il+1];				//
			#local q2 = (ppp[Il+1] + ppp[Il+2])/2;	//
			#local p = quad(aa, q0, q1, q2);	//
		#break
		#case (3)								// 	cas cubique:
			#local q0 = (ppp[Il] + 4*ppp[Il+1] +   ppp[Il+2]          )/6;
			#local q1 = (         4*ppp[Il+1] + 2*ppp[Il+2]          )/6;
			#local q2 = (         2*ppp[Il+1] + 4*ppp[Il+2]          )/6;
			#local q3 = (           ppp[Il+1] + 4*ppp[Il+2] + ppp[Il+3])/6;
			#local p = cub(aa, q0, q1, q2, q3);
		#break
	#end
	p;
#end


